
/*
      _______                          .__ .__   .__   .__                 
      \      \    ____   __ __ _______ |__||  |  |  |  |__|  ____    ____  
      /   |   \ _/ __ \ |  |  \\_  __ \|  ||  |  |  |  |  | /  _ \  /    \ 
     /    |    \\  ___/ |  |  / |  | \/|  ||  |__|  |__|  |(  <_> )|   |  \
     \____|__  / \___  >|____/  |__|   |__||____/|____/|__| \____/ |___|  /
    =========\/======\/=================================================\/==
  v0.01 09/NOV/2007   Copyright 2007-2007 Scott D. Yelich SOME RIGHTS RESERVED
  ,-*~'`^`'~*-,._.,-*~'`^`'~*-,._.,-*~'`^`'~*-,._.-*~'`^`'~*-,._.,-*~'`^`'~*-,


  LICENSE:  Creative Commons Attribution 3.0 License.
  SEE:      http://creativecommons.org/licenses/by/3.0/


  Fri Nov  9 23:00:32 EST 2007, v0.01 sdy
 
  This file is part of CommandLine

*/

#include "CommandLine.h"

CommandLine::CommandLine(bool strict_dashes, std::string const & pre) :
  Cache(),
  prefix(pre),
  strict(strict_dashes)
{
}

CommandLine::~CommandLine()
{
}

std::string
CommandLine::program()
{
  return progname;
}

int
CommandLine::words(std::string const & opt)
{
  int wc = 0;
  std::string s = get(opt);
  bool inword = false;
  if (s.size() > 0) {
    wc++;
    int len = s.size();
    for (int i = 0; i<len; i++) {
      if (' ' == s.at(i)) { // isspace?
        if (inword) {
          wc++;
        }
        inword = false;
      } else {
        inword = true;
      }
    }
  }
  return wc;
}

int
CommandLine::stringToInt(std::string const & s)
{
  std::istringstream i(s);
  int x;
  if (!(i >> x))
    throw TypedException<CommandLine::Errors>(CommandLine::InvalidParameter, "can't convert '" + s + "' to int!");
  return x;
}

std::string
CommandLine::intToString(int i)
{
  std::stringstream s;
  s << i;
  return s.str();
}

void
CommandLine::process(int argc, char * argv[], std::string const & format)
{

  //  do nothing if no format string (no throw)
  if (0 == format.size()) { return; }

  //
  //  read in format string and store in StrStrMap
  //  

  std::stringstream ss(format);

  int i;
  std::string s;
  
  while (ss.good()) {
    ss >> s;
    ss >> i;
    set(prefix+s, intToString(i));
  }

  //  save initial values

  this->argc = argc;
  this->argv = argv;
  this->progname  = *argv;

  //  skip name of program ...
  --argc;
  ++argv;

  //  
  //  Loop over command line options and process ...
  //  

  std::string opt;
  std::string key;
  std::string value;

  while (argc > 0) {

    //  get current option and decrease count of options remaining
    --argc;
    opt = *argv;
    ++argv;
    key = prefix + opt;

    //  if key exists, process required number of parameters
    if (exists(key)) {
      i = stringToInt(get(key));
      //  are there enough options remaining?
      if (argc >= i) {
        //  reduce count by option required amount, add options to key value
        argc -= i;
        value.clear();
        //  construct string of required options, error if any missing
        int j = i;
        // additional arguments may *not* start with a '-' ...
        while (j > 0 && (!strict || (strict && ('-' != argv[0][0])))) {
          if (value.size() > 0) {
            value.append(" ");
          }
          value.append(*argv);
          ++argv;
          --j;
        }
        if (0 == j ) {
          //  save the params for the option 
          set(opt, value);
        } else {
          std::stringstream ss;
          ss <<  "Option '" << opt << "' requires " << j << " additional argument(s).";
          throw TypedException<CommandLine::Errors>(CommandLine::RequiresParameter, ss.str());
        }
      } else {
	std::stringstream ss;
        ss <<  "Option '" << opt << "' requires " << i << " argument(s).";
        throw TypedException<CommandLine::Errors>(CommandLine::RequiresParameter, ss.str());
      }
    }  else {
      //  key does not exist, so add words to key "--"
      value = "";
      if (exists("--")) {
        value.append(get("--"));
      }
      //  not "--" means stray word ...
      if ("--" != opt) {
        if (value.size() > 0){
          value.append(" ");
        }
        value.append(opt); // not key (as it contains prefix)
      } else {
        //  is "--" means slurp remaining words ...
        if (argc > 0) {
          if (value.size() > 0){
            value.append(" ");
          }
          while (argc > 0) {
            value.append(*argv);
            --argc;
            if (argc > 0) {
              value.append(" ");
              ++argv;
            }
          }
        }
      }
      set("--", value);
    }
  }
}

